#!/usr/bin/env python
from utils import read_config, Realm, Server
from flask import Flask, Response
from datetime import datetime
from dateutil import tz
from email.utils import formatdate
from time import mktime

import logging
log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)

config = read_config('mdsigner.yaml')
app = Flask(__name__)
server = Server()


@app.route('/<realm>/entities',
           strict_slashes=False,
           methods=['GET'])
def serve_all(realm):
    print(f"all in {realm}")
    response = Response()
    response.headers['Content-Type'] = "application/samlmetadata+xml"
    response.headers['Content-Disposition'] = "filename = \"metadata.xml\""

    data = server[realm].all_entities()
    response.data = data.md
    max_age = int((data.valid_until -
                   datetime.now(tz.tzutc())).total_seconds())

    response.headers['Cache-Control'] = f"max-age={max_age}"
    response.headers['Last-Modified'] = formatdate(
        timeval=mktime(data.last_modified.timetuple()),
        localtime=False,
        usegmt=True)
    return response


@app.route('/<realm>/entities/<path:entity_id>',
           strict_slashes=False,
           methods=['GET'])
def serve_one(realm, entity_id):
    print(f"entity_id: {entity_id}")
    response = Response()
    response.headers['Content-Type'] = "application/samlmetadata+xml"
    response.headers['Content-Disposition'] = "filename = \"metadata.xml\""

    try:
        data = server[realm][entity_id]
        response.data = data.md
        max_age = data.max_age
        last_modified = data.last_modified
    except Exception:
        response.data = "No valid metadata\n"
        response.headers['Content-type'] = "text/html"
        response.status = 404
        max_age = 60
        last_modified = datetime.now(tz.tzutc())

    response.headers['Cache-Control'] = f"max-age={max_age}"
    response.headers['Last-Modified'] = formatdate(
        timeval=mktime(last_modified.timetuple()),
        localtime=False,
        usegmt=True)
    return response


for realm, values in config.items():
    print(f"realm: {realm}")
    location = values['metadir']
    signer = values['signer']
    server[realm] = Realm(location, signer)

if __name__ == "__main__":
    app.run(host='127.0.0.1', port=5001, debug=False)

application = app
